#BTLE delegate class

from bluepy import btle
import binascii
from Device import BLEDevice
import os
from threading import Timer
from utils import playAudio

global dev_list
dev_list = None

def connect(mac):
    """Connects to a specified BLE MAC address"""
    global dev_list
    
    if(dev_list == None):
        print("[BLECtl] Initializing device list..")
        dev_list = list()

    print("[BLECtl] Connecting to "+str(mac)+"...")
    try:
        #Connect to device and retrieve information
        dev_append = None
        newDev = btle.Peripheral(mac)
        char = newDev.getCharacteristics(uuid=btle.AssignedNumbers.deviceName)
        dev_name = char[0].read()
        dev_append = BLEDevice(dev_name, newDev)
        dev_list.append(dev_append)
        print("[BLECtl] Connection with "+str(dev_name)+" successful.")
        playAudio("/home/linaro/google_coffee_machine_demo/DB/ble-ctl/dev_connected.wav", True)
        return True
    except Exception as ex:
        print("[ERROR] Exception > "+ex.message)
        if(dev_append != None):
            dev_list.remove(dev_append)
        return False

def listDevices():
    """Prints all connected devices"""
    global dev_list

    if(len(dev_list) == None):
        print("[BLECtl] No devices in list.")
    else:
        print("[BLECtl] Connected devices:")
        count = 1
        for dev in dev_list:
            print(">"+str(count)+" : "+dev.name)
            count = count + 1
        print("===============")

def getDevice(name):
    """Retrieves a BLEDevice from connected devices list"""
    global dev_list
    if(dev_list != None):
#        print("[BLECtl] No devices connected yet.")
#    else:
        for dev in dev_list:
            if(dev.name == name):
                return dev
        print("[ERROR][BLECtl] Device "+name+" not found. Is it connected?")
        return False

def functionWatchdog():
    print("[PBClient][DEBUG] Timer is still alive. Rebooting software...")
    playAudio("/home/linaro/google_coffee_machine_demo/DB/ble-ctl/error.wav", True)
    os._exit(1)

def updateNotifications(timeout):
    """Checks notification messages on all connected devices. Should be called on a loop"""
    global dev_list
    if(dev_list != None):
        #print("[BLECtl] No devices connected yet.")
    #else:
        for dev in dev_list:
            if(dev.hasNotify):
                try:
                    t = Timer(11, functionWatchdog)
                    t.start()
                    dev.peripheral.waitForNotifications(timeout)
                    t.cancel()
                except Exception as ex:
                    print("[ERROR] Error while updating BLE notifications!")
                    if((ex.message.find("object has no attribute") != -1) or (ex.message.find("Helper not started") != -1)):
                        print(ex.message)
                        print("Fatal error. Exiting...")
                        playAudio("/home/linaro/google_coffee_machine_demo/DB/ble-ctl/error.wav", True)
                        os._exit(1)
                    else:
                        print(ex.message)
