#Device class
from bluepy import btle
from binascii import hexlify
from binascii import unhexlify
import os
from utils import playAudio

SUBSCRIBE_SVC = "\x01\x00"
UNSUBSCRIBE_SVC = "\x00\x00"

class BLEDevice():
    """Abstracts access to underlying Bluepy's peripheral interface"""
    def __init__(self, name_dev, periph):
        self.name = name_dev
        self.peripheral = periph
        self.hasNotify = False
        self._chars = self.peripheral.getCharacteristics()


    def subscribe(self, char_name, callback):
        """Enables notification for a given device characteristic"""
        found = False
        for chars in self._chars:
            if((chars.uuid.getCommonName() == char_name) or (hexlify(chars.uuid.binVal) == char_name)):
                if(self.hasNotify == False):
                    self.peripheral.setDelegate(CustomDelegateClass(callback))
                self.peripheral.writeCharacteristic(chars.valHandle+1, SUBSCRIBE_SVC)
                found = True
                self.hasNotify = True
        if(found):
            #print("[BLECtl] Subscription added successfully.")
            pass
        else:
            print("[BLECtl] ERROR: Characteristic not found on device "+self.name)

    def readFromCharacteristic(self, char_name):
        """Attempts to read a value from a characteristic name or UUID"""
        for chars in self._chars:
            if((chars.uuid.getCommonName() == char_name) or (hexlify(chars.uuid.binVal) == char_name)):
                try:
                    return chars.read()
                except Exception as ex:
                    print("["+self.name+"][ReadChar]"+"[ERROR] Exception > "+ex.message)
                    if((ex.message.find("object has no attribute") != -1) or (ex.message.find("Helper exited") != -1) or (ex.message.find("did you call connect"))):
                        playAudio("/home/linaro/google_coffee_machine_demo/DB/ble-ctl/error.wav", True)
                        print("Fatal error. Exiting...")
                        os._exit(1)
                    else:
                        print(ex.message)
                    return False
        return False

    def getCharacteristics(self):
        """Returns all characteristics from a BLE device"""
        char_list = list()
        for chars in self._chars:
            char_list.append({'name':chars.uuid.getCommonName(), 'uuid':hexlify(chars.uuid.binVal), 'handle':chars.getHandle()})
        return char_list
        
    def writeToCharacteristic(self, char_name, value):
        """Writes a value to a given characteristic name or UUID"""
        for chars in self._chars:
            if((chars.uuid.getCommonName() == char_name) or (hexlify(chars.uuid.binVal) == char_name)):
                try:
                    if(isinstance(value, int)):
                        return chars.write(value)
                    else:
                        return chars.write(unhexlify(value))
                except Exception as ex:
                    print("["+self.name+"][WriteChar]"+"[ERROR] Exception > "+ex.message)
                    if((ex.message.find("object has no attribute") != -1) or (ex.message.find("Helper exited") != -1) or (ex.message.find("did you call connect"))):
                        print("Fatal error. Exiting...")
                        playAudio("/home/linaro/google_coffee_machine_demo/DB/ble-ctl/error.wav", True)
                        os._exit(1)
                    else:
                        print(ex.message)
                    return False
        return False
    
        


class CustomDelegateClass(btle.DefaultDelegate):
    def __init__(self, callback):
        btle.DefaultDelegate.__init__(self)
        self.handleNotification = callback

