from google.cloud import pubsub_v1
from google.cloud import storage
import BluCallbacks
import utils
import lampHandler
import logging
import json
import time
import datetime
from coffeeMachineHandler import getInstance
import storage


UTC_FORMAT = "%Y-%m-%dT%H:%M:%S.00Z"
bleonline = False

class PBClient:
    def __init__(self):
        self.publisher = None
        self.subscriber = None
        self.projectid = None

def getClient(projectId):
    print("[PBCtl] Creating pubsub client for project ID "+str(projectId))
    #Creates a Publisher and a Subscriber and puts in PBClient object
    client = PBClient()
    logging.basicConfig()

    #WARNING: Do not use explicit credential declaration below!
    client.publisher = pubsub_v1.PublisherClient()
    client.subscriber = pubsub_v1.SubscriberClient()
    client.projectId = projectId
    return client

client = getClient("db410c-dev-ga04-233012")

def set_bleonline_status(status):
    global bleonline
    bleonline = status

def get_utc_timestamp_from_string(stringosa):
    return datetime.datetime.strptime(stringosa, UTC_FORMAT)

def get_utc_timestamp(seconds=None):
    return time.strftime("%Y-%m-%dT%H:%M:%S.00Z", time.gmtime(seconds))

def subscribeTopics(client):
    #Topics (Only subscribe to subscription topics set in GCP!)
    TOPIC_DEVICECTL_SUB = 'projects/{project_id}/subscriptions/BluDevSub'.format(project_id=client.projectId)
    
    #Subscription routine
    fut = client.subscriber.subscribe(TOPIC_DEVICECTL_SUB, deviceControlCallback)
    print("[PBCtl] Subscription complete.")

def deviceControlCallback(message):
    global bleonline
    message.ack()
    msg_json = json.loads(message.data)
    isOld = False
    print("[PBClient] Message: "+str(message.data))

    if(bleonline == True):

        if('time' in msg_json):
            timecheck = get_utc_timestamp_from_string(msg_json['time'])
            timenow = datetime.datetime.utcnow()
            age_seconds = (timenow-timecheck).total_seconds()
            if(age_seconds > 8):
                print("[PBClient]Throwing away old message!")
                isOld = True

        if ('cmd' in msg_json) and (isOld == False):
            #print("New device control request")
            if(msg_json['cmd'] == 'on'):
                if(msg_json['dev'] == 'coffee'):
                    print("Coffee machine [ON] request")
                    BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'01')
                else:
                    print("Lamp [ON] request")
                    lampHandler.setBrightness(None, BluCallbacks.getDevice("Rq Ble Light"))
            elif(msg_json['cmd'] == 'off'):
                if(msg_json['dev'] == 'coffee'):
                    print("Coffee machine [OFF] request")
                    BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'00')
                else:
                    print("Lamp [OFF] request")
                    lampHandler.setBrightness(0, BluCallbacks.getDevice("Rq Ble Light"))
            elif(msg_json['cmd'] == "temp"):
                print("Lamp temp")
                lampHandler.setTemp(int(msg_json['temp']), BluCallbacks.getDevice("Rq Ble Light"))
            elif(msg_json['cmd'] == 'brightness'):
                print("Lamp brightness")
                lampHandler.setBrightness(int(msg_json['brightness']), BluCallbacks.getDevice("Rq Ble Light"))
            elif(msg_json['cmd'] == 'shortCoffee'):
                print("Coffee Machine [ShortCoffee] request")
                BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'02')
            elif(msg_json['cmd'] == 'longCoffee'):
                print("Coffee Machine [LongCoffee] request")
                BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'03')
            elif(msg_json['cmd'] == 'status'):
                if(msg_json['dev'] == 'coffee'):
                    print("Coffee Machine [Status] request")
                    #BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'07')
                    cm = getInstance()
                    #print("Uploaded coffee machine status: ", cm.toJSON())
                    client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectId), cm.toJSON().encode('utf-8'))
    else:
        print("[PBCtl] Man! Bluetooth devices are offline!")

#TODO: Subscriptions should be separated by GCP project ID
