#! /usr/bin/env python

from bluepy import btle
from time import sleep
import BluCallbacks
import utils

def onCoffeeDoneCallback(type):
    print("[Coffee] Coffee is ready!")

def coffeeMachineCallback(cHandle, data):
    """Coffee machine responds when data is written on HeartRateControlPoint"""
    if(cHandle == 11):
        split_data = utils.unpackData(data)
        print("Coffee machine has sent dataframe >>", split_data)

        #Check state accordingly
        if(len(split_data) == 4):
            print("All sensor update")
            #2nd nibble // Power
            if(split_data[1] == 1):
                print("-> Coffee machine is ON")
            else:
                print("-> Coffee machine is OFF")
            #3rd nibble // Water level
            if(split_data[2] == 1):
                print("-> Water level is GOOD")
            else:
                print("-> Water level is BAD")
            #4th nibble // Glass position
            if(split_data[3] == 1):
                print("-> Glass positioning is GOOD")
            else:
                print("-> Glass positioning is BAD or NO GLASS")
        elif(split_data[0] == 0):
            print("Turn off")
        elif(split_data[0] == 1):
            print("Turn on")
        elif(split_data[0] == 2):
            if(split_data[1] == 3):
                print("Short coffee [DOING]")
            elif(split_data[1] == 1):
                print("Short coffee [READY]")
                onCoffeeDoneCallback("Short")
        elif(split_data[0] == 3):
            if(split_data[1] == 3):
                print("Long coffee [DOING]")
            elif(split_data[1] == 1):
                print("Long coffee [READY]")
                onCoffeeDoneCallback("Long")
        elif(split_data[0] == 4):
            print("Water level")
        elif(split_data[0] == 5):
            print("Coffee level")

            #Une os dois valores
            res = (split_data[2] << 8)+split_data[1]
            res = (res/2)/29.1
            res = round(res, 3)

            porcentagem = res/11.787
            porcentagem = round(((1 - porcentagem)*100), 1)
            if(porcentagem > 100):
                porcentagem = 100
            if(porcentagem < 0):
                porcentagem = 0
            
            print("> Porcentagem de cafe: " + str(porcentagem)+"%")
            print("> Distancia: "+str(res)+"cm")
        elif(split_data[0] == 6):
            print("Glass positioning")

def main():
    print("Execution started")

    #Connect to device
    connected = BluCallbacks.connect("00:02:5B:00:15:10")
    while(not connected):
        connected = BluCallbacks.connect("00:02:5B:00:15:10")

    #List connected devices
    BluCallbacks.listDevices()

    #Subscribe to a notification char with callback
    BluCallbacks.getDevice("CSR HR Sensor").subscribe("Heart Rate Measurement", coffeeMachineCallback)
    BluCallbacks.getDevice("CSR HR Sensor").subscribe("Battery Level", coffeeMachineCallback)
    
    #Read information from a device
    print(BluCallbacks.getDevice("CSR HR Sensor").readFromCharacteristic("Device Name"))
    
    #Get all characteristics and UUIDs of a device
    listosa = BluCallbacks.getDevice("CSR HR Sensor").getCharacteristics()
    for item in listosa:
        print(item['name'], item['uuid'], item['handle'])

    time = 1.7
    while True:
        #BluCallbacks.updateNotifications(3)
        #print(">>Turn on")
        #BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'01')
        #BluCallbacks.updateNotifications(3)
        #sleep(time/2)
        #print(">>Turn off")
        #BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'00')
        #BluCallbacks.updateNotifications(3)
        #sleep(time)
        print(">>Coffee status")
        BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'05')
        BluCallbacks.updateNotifications(3)
        sleep(time)
        #print(">>Coffee level")
        #BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'05')
        #BluCallbacks.updateNotifications(3)
        #sleep(time)
        #print(">>Short coffee")
        #BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'02')
        #count = 15
        #while(count > 0):
        #    BluCallbacks.updateNotifications(2)
        #    sleep(0.2)
        #    count = count - 1

if __name__ == "__main__":
    main()

#TODO: No timeout in BT connection