import utils, json, time, datetime
import storage
class CoffeeMachine:
    def __init__(self):
        self.isOn = False
        self.waterLevel = None
        self.hasGlass = None
        self.busy = False
        self.coffeeLevel = 0.0
    def toJSON(self):
        state = dict()
        state['isOn'] = self.isOn
        state['waterLevel'] = self.waterLevel
        state['hasGlass'] = self.hasGlass
        state['busy'] = self.busy
        state['time'] = get_utc_timestamp()
        state['coffeeLevel'] = self.coffeeLevel
        return json.dumps(state)

coffeeMachine = CoffeeMachine()

def get_utc_timestamp(seconds=None):
    return time.strftime("%Y-%m-%dT%H:%M:%S.00Z", time.gmtime(seconds))

def getInstance():
    return coffeeMachine

def onCoffeeDoneCallback(coffeetype):
    print(str(coffeetype)+" coffee is [DONE]")

def coffeeMachineCallback(cHandle, data):
    """Coffee machine responds when data is written on HeartRateControlPoint"""
    if(cHandle == 11):
        split_data = utils.unpackData(data)
        #print("Coffee machine has sent dataframe >>", split_data)

        #Check state accordingly
        if(len(split_data) == 4):
            #print("[coffeeHandler] All sensor update")
            #2nd nibble // Power
            if(split_data[1] == 1):
                #print("-> Coffee machine is ON")
                coffeeMachine.isOn = True
            else:
                #print("-> Coffee machine is OFF")
                coffeeMachine.isOn = False
            #3rd nibble // Water level
            if(split_data[2] == 1):
                #print("-> Water level is GOOD")
                coffeeMachine.waterLevel = True
            else:
                #print("-> Water level is BAD")
                coffeeMachine.waterLevel = False
            #4th nibble // Glass position
            if(split_data[3] == 1):
                #print("-> Glass positioning is GOOD")
                coffeeMachine.hasGlass = True
            else:
                #print("-> Glass positioning is BAD or NO GLASS")
                coffeeMachine.hasGlass = False
        elif(split_data[0] == 0):
            #print("[coffeeHandler] Turn off")
            coffeeMachine.isOn = False
        elif(split_data[0] == 1):
            #print("[coffeeHandler] Turn on")
            coffeeMachine.isOn = True
        elif(split_data[0] == 2):
            if(split_data[1] == 3):
                coffeeMachine.busy = True
                #print("[coffeeHandler] Short coffee [DOING]")
            elif(split_data[1] == 1):
                coffeeMachine.busy = False
                #print("[coffeeHandler] Short coffee [READY]")
                onCoffeeDoneCallback("Short")
        elif(split_data[0] == 3):
            if(split_data[1] == 3):
                coffeeMachine.busy = True
                #print("[coffeeHandler] Long coffee [DOING]")
            elif(split_data[1] == 1):
                coffeeMachine.busy = False
                #print("[coffeeHandler] Long coffee [READY]")
                onCoffeeDoneCallback("Long")
        elif(split_data[0] == 4):
            #print("[coffeeHandler] Water level")
            if(split_data[1] == 1):
                coffeeMachine.waterLevel = True
            else:
                coffeeMachine.waterLevel = False
        elif(split_data[0] == 5):
            print("[coffeeHandlerBLE] Coffee level")

            #Une os dois valores
            res = (split_data[2] << 8)+split_data[1]
            res = (res/2)/29.1
            res = round(res, 3)

            porcentagem = res/11.787
            porcentagem = round(((1 - porcentagem)*100), 1)
            if(porcentagem > 100):
                porcentagem = 100.0
            if(porcentagem < 0):
                porcentagem = 0.0
            coffeeMachine.coffeeLevel = porcentagem
            print("> Porcentagem de cafe: " + str(porcentagem)+"%")
        elif(split_data[0] == 6):
            #print("[coffeeHandler] Glass positioning")
            if(split_data[1] == 1):
                coffeeMachine.hasGlass = True
            else:
                coffeeMachine.hasGlass = False

    #Send status to cloud
    storage.update_coffee_status(coffeeMachine.toJSON())