#! /usr/bin/env python

from time import sleep
import BluCallbacks
import PBClient
import math
import json
from coffeeMachineHandler import coffeeMachineCallback, getInstance
cm = getInstance()
import lampHandler
import storage
import os
from utils import playAudio

def main():
    print("Execution started")

    #Creating PubSub client
    client = PBClient.getClient("db410c-dev-ga04-233012")
    PBClient.subscribeTopics(client)

    PBClient.set_bleonline_status(False)
    storage.update_ble_status('{"ble": false}')
    playAudio("/home/linaro/google_coffee_machine_demo/DB/ble-ctl/begun_connection.wav", True)

    #TODO: Make sure controller initialization goes through if a device is offline
    #Connect to lamp and authenticate
    conn_successful = BluCallbacks.connect("98:5d:ad:18:f7:18")
    count = 0
    while(conn_successful == False):
        sleep(1)
        count += 1
        conn_successful = BluCallbacks.connect("98:5d:ad:18:f7:18")

        if(count > 15):
            print("[BleCtl][ERROR] Adapter is problematic! Rebooting...")
            sleep(0.5)
            playAudio("/home/linaro/google_coffee_machine_demo/DB/ble-ctl/error.wav", True)
            sleep(0.4)
            os.system("reboot")
    lamp = BluCallbacks.getDevice("Rq Ble Light")
    count = 0
    lampHandler.authenticateConnection(lamp)
    
    #Connect to mock Coffee Machine CSR
    conn_successful = BluCallbacks.connect("00:02:5b:00:15:10")
    while(conn_successful == False):
        sleep(1)
        conn_successful = BluCallbacks.connect("00:02:5b:00:15:10")
    coffeeMachine = BluCallbacks.getDevice("CSR HR Sensor")
    coffeeMachine.subscribe("Heart Rate Measurement", coffeeMachineCallback)
    coffeeMachine.subscribe("Battery Level", coffeeMachineCallback)
    sleep(0.1)
    coffeeMachine.writeToCharacteristic("Heart Rate Control Point", b'07')
    sleep(0.3)

    PBClient.set_bleonline_status(True)
    storage.update_ble_status('{"ble": true}')

    #Lamp example JSON payloads
    payload = {
        'cmd': 'on',
        'dev': 'lamp'
    }
    payload2 = {
        'cmd': 'off',
        'dev': 'lamp'
    }
    payload3 = {
        'cmd': 'temp',
        'temp': 5800
    }
    payload4 = {
        'cmd': 'temp',
        'temp': 2700
    }
    payload5 = {
        'cmd': 'brightness',
        'brightness': 5
    }
    payload6 = {
        'cmd': 'brightness',
        'brightness': 13
    }

    #Coffee machine JSON payloads
    payload7 = {
        'cmd': 'shortCoffee'
    }
    payload8 = {
        'cmd': 'longCoffee'
    }
    payload9 = {
        'cmd': 'on',
        'dev': 'coffee'
    }
    payload10 = {
        'cmd': 'off',
        'dev': 'coffee'
    }

    print("Updating notifications")
    BluCallbacks.updateNotifications(2)
    playAudio("/home/linaro/google_coffee_machine_demo/DB/ble-ctl/connected.mp3", False)

    #Define test pace
    #sleep(7)
    print("Starting loop")
    coffeeUpdateCount = 0
    while True:
        #BluCallbacks.getDevice("CSR HR Sensor").writeToCharacteristic("Heart Rate Control Point", b'05')
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload))
        #sleep(time/2)
        #BluCallbacks.updateNotifications(time)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload2))
        #sleep(time/2)
        #BluCallbacks.updateNotifications(time)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload3))
        #sleep(time/2)
        #BluCallbacks.updateNotifications(time)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload4))
        #sleep(time/2)
        #BluCallbacks.updateNotifications(time)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload6))
        #sleep(time/2)
        #BluCallbacks.updateNotifications(time)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload5))
        #sleep(time/2)
    
        #Coffee machine publishes
        #BluCallbacks.updateNotifications(time)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload9))
        #BluCallbacks.updateNotifications(time)
        #sleep(time/2)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload7))
        #BluCallbacks.updateNotifications(time)
        #sleep(time*4)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload8))
        #BluCallbacks.updateNotifications(time)
        #sleep(time*6)
        #client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectid), json.dumps(payload10))
        #BluCallbacks.updateNotifications(time)
        coffeeUpdateCount += 1
        BluCallbacks.updateNotifications(3)
        sleep(0.5)
        if(coffeeUpdateCount > 1):
            coffeeMachine.writeToCharacteristic("Heart Rate Control Point", b'05')
            coffeeUpdateCount = 0
        sleep(2)

if __name__ == "__main__":
    main()
