from google.cloud import pubsub_v1
import logging
import utils
import json
import time, datetime
import storage

ZBSERVER_ENDPOINT = "http://localhost:4000/"

class PBClient:
    def __init__(self):
        self.publisher = None
        self.subscriber = None
        self.projectid = None

UTC_FORMAT = "%Y-%m-%dT%H:%M:%S.00Z"

def getClient(projectId):
    print("[PBCtl] Creating pubsub client for project ID "+str(projectId))
    #Creates a Publisher and a Subscriber and puts in PBClient object
    client = PBClient()
    logging.basicConfig()
    #WARNING: Do not use explicit credential declaration below!
    client.publisher = pubsub_v1.PublisherClient()
    client.subscriber = pubsub_v1.SubscriberClient()
    client.projectid = projectId
    return client

def get_utc_timestamp_from_string(stringosa):
    return datetime.datetime.strptime(stringosa, UTC_FORMAT)

def get_utc_timestamp(seconds=None):
    return time.strftime("%Y-%m-%dT%H:%M:%S.00Z", time.gmtime(seconds))

def subscribeTopics(client):
    #Topics (Only subscribe to subscription topics set in GCP!)
    TOPIC_DEVICECTL_SUB = 'projects/{project_id}/subscriptions/ZigCtlSub'.format(project_id=client.projectid)
    
    #Subscription routine
    fut = client.subscriber.subscribe(TOPIC_DEVICECTL_SUB, deviceControlCallback)

    print("[PBCtl] Subscription complete.")

def checkDeviceStatus():
    print("[PBCallbacks] Polling ZBServer for device status")
    response = utils.doGetRequest(ZBSERVER_ENDPOINT+"deviceState")
    deviceState = response['message']
    if(len(deviceState) == 0):
        print("> No devices yet.")
        storage.update_zig_status(json.dumps([{'lamp': False}, {'lock': False}]))
        return [{'lamp': False}, {'lock': False}]
    else:
        lock_online = False
        lamp_online = False
        for device in deviceState:
            if(str(device['devId']) != "LOCKENDPOINT"):
                print("Dev "+str(device['devId'])+" status: "+str(device['online']))
                lamp_online = device['online']
            elif(lock_online == False):
                print("Dev "+str(device['devId'])+" status: "+str(device['online']))
                lock_online = device['online']
        print("[PBCallbacks] Uploading zigbee status...")
        output = [{'lamp': lamp_online}, {'lock': lock_online}]
        storage.update_zig_status(json.dumps(output))
        return output


def deviceControlCallback(message):
    msg_json = json.loads(message.data)
    isOld = False
    print("[PBCallbacks Zigbee] Message: "+str(message.data))

    if('time' in msg_json):
        print("[PbCallbacks] Found timestamp")
        timecheck = get_utc_timestamp_from_string(msg_json['time'])
        timenow = datetime.datetime.utcnow()
        age_seconds = (timenow-timecheck).total_seconds()
        if(age_seconds > 8):
            print("[PbCallbacks] Throwing away old message!")
            isOld = True
            
    if (('cmd' in msg_json) and (isOld == False)):
        #if(msg_json['cmd']):
        print("New device control request")
        if(msg_json['cmd'] == 'unlock'):
            print("Unlocking the door")
            resp = utils.doPostRequest("", ZBSERVER_ENDPOINT+"unlockDoor", None)
        elif(msg_json['cmd'] == 'lock'):
            print("Locking the door")
            resp = utils.doPostRequest("", ZBSERVER_ENDPOINT+"lockDoor", None)
        elif(msg_json['cmd'] == 'lightoff'):
            print("Turn off light")
            extra_args = { 'devId': 48042}
            resp = utils.doPostRequest("", ZBSERVER_ENDPOINT+"turnOff", extra_args)
        elif(msg_json['cmd'] == 'lighton'):
            print("Turn on light")
            extra_args = { 'devId': 48042}
            resp = utils.doPostRequest("", ZBSERVER_ENDPOINT+"turnOn", extra_args)
        print(resp)
    message.ack()

#TODO: Subscriptions should be separated by GCP project ID