#! /usr/bin/env python

import json, os, subprocess
from time import sleep
from threading import Thread
import jobcontroller
import ledcontroller as lc

global jobQueue
global device_id
device_id = "GA-DB04"

def main():
    lc.init()

    wLanState = ""
    cont = 0
    time_limit = 360

    while(wLanState != "up" and cont <= time_limit):
        try:
            wLanState = subprocess.check_output("cat /sys/class/net/wlan0/operstate", shell=True)
            wLanState = wLanState[:-1]
            cont = cont + 1
            print("WLAN0 STATE " + str(cont) + ": " + str(wLanState))
            sleep(0.1)
        except (OSError, subprocess.CalledProcessError):
            lc.TurnOn(24)
            print("[SysWatch] wlan0 interface is not ready.")
    sleep(0.3)
    if cont >=time_limit:
        print("[SysWatch] There is no known Wireless Network. Starting Hotspot.")
        lc.TurnOn(24) #Turn on Red Led.
        os.system("play /home/linaro/google_coffee_machine_demo/DB/no_internet.wav")
        os.system("bash /home/linaro/google_coffee_machine_demo/DB/hotspot_service/hotspot_setup.sh")
    else:
        #internetState = subprocess.check_output("bash /home/linaro/google_coffee_machine_demo/DB/syswatch_dev/hasInternet.sh", shell=True)
        internetState = subprocess.check_output("ping -c 3 8.8.8.8 | tail -2 | head -1 | awk '{print $6}'", shell=True).rstrip()
        #internetState = internetState[:-1]

        print("[SysWatch] Internet has " + str(internetState)+ " packet loss.")

        if (internetState == "0%") or (internetState == "33.3333%") or (internetState == "66.6667%"):
            print("[SysWatch] The Wireless Network has internet connection.")
            lc.TurnOff(24) #Turn off Red Led.
            os.system("play /home/linaro/google_coffee_machine_demo/DB/has_internet.wav")
        elif (internetState == "") or (internetState == "100%"):
            print("[SysWatch] The Wireless Network does not have wireless connection. Starting Hotspot.")
            lc.TurnOn(24) #Turn on Red Led.
            os.system("play /home/linaro/google_coffee_machine_demo/DB/no_internet.wav")
            os.system("bash /home/linaro/google_coffee_machine_demo/DB/hotspot_service/hotspot_setup.sh")
    
    print("SysWatch initialized")    
    print("Device: "+device_id)

    #Load all apps to be monitored from disk
    jobQueue = jobcontroller.loadJobs()
    
    #Run jobs as separate processes
    jobcontroller.runJobs(jobQueue)

    #Debug
    sect_count = 14
    sleep(0.3)

    #Watch for processes quitting
    while(True):
        sect_count = sect_count + 1
        for job in jobQueue:
            
            if(sect_count > 16):
                print("[SysWatch] Job ["+job.name+"] status: "+str(job.process.returncode))

            if(job.process.poll() != None):
                #Send MQTT error report
                #createErrorReport(job, MQTTClient)
                #createReportStatus(jobQueue)

                if(job.restart == True):
                    print("[SysWatch] Job ["+job.name+"] has died. Restarting...")
                    sleep(0.2)
                    print("* =-=-=-=-=-=-=-=-= Tail of logfile:")
                    sleep(0.1)
                    print(jobcontroller.getLastLog(job, 13))
                    sleep(0.1)
                    print("* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=\n")
                    jobQueue.remove(job)
                    jobQueue.append(jobcontroller.runSingleJob(job, True))
                else:
                    print("[SysWatch] Job ["+job.name+"] has died.")
                    print("\n=-=-=-=-=-=-=-=-= Tail of logfile:")
                    print(jobcontroller.getLastLog(job, 5))
                    print("=-=-=-=-=-=\n")
                    jobQueue.remove(job)
        
        if(sect_count > 16):
            print("")
            sect_count = 0
            createReportStatus(jobQueue)
        sleep(0.5)
    
    print("Execution ended.")

def createReportStatus(jobQueue):
    f_rep = open("output_rep.txt", 'w')
    status_type = "report"
    bld = "SysWatch report:\n"
    bld = bld + "Device: "+device_id+'\n'
    bld = bld + "JobList: =-=-=-=-=-=-=\n"
    jobs = list()
    for job in jobQueue:
        if(job.process != None):
            if(job.process.poll() == None):
                jobs.append({'status' : 'running', 'name': job.name})
                bld = bld + "Job ["+job.name+"] is running.\n"
            else:
                bld = bld + "Job ["+job.name+"] is down with exitcode "+str(job.process.returncode)+".\n"
    bld2 = json.dumps({'status': status_type, 'jobs':jobs})
    f_rep.write(bld)
    f_rep.close()
        
def createErrorReport(job, cli):
    bld = "SysWatch ERROR report: \n"
    bld = bld + "Device: "+device_id+'\n'
    bld = bld + "Error: Job ["+job.name+"] has died with return code "+str(job.process.returncode)+'\n'
    bld = bld + "StackTrace: =====\n"
    bld = bld + jobcontroller.getLastLog(job, 5) + '\n'
    bld = bld + "=========\n"
    if(job.restart):
        bld = bld + "Re-starting job ["+job.name+"]\n"
    cli.publish("Tower/Error", bld, 0)

def initMQTT():
    #try:
        print("[syswatch] Creating MQTT client..")
        MQTTCli = MQTTConfig.createClient()
        print("[syswatch] Connecting to IoT center..")
        MQTTCli.connect()
        print("[syswatch] Subscribing to topics..")
        MQTTCallbacks.subscribeTopics(MQTTCli)
        print("[syswatch] MQTT succesfully initialized.")
        return MQTTCli
    #except Exception as er:
    #    print("[syswatch] MQTT setup error: "+str(er.message))
    #    return False

    

if __name__=="__main__":
    main()