//Zigbee related libraries
var ZShepherd = require('zigbee-shepherd');
var zserver = new ZShepherd('/dev/ttyACM0');
const { exec } = require('child_process');

//Lamp State var
var lampState = "ON";

//REST api library
var express = require('express');
var rest = express();
const corsOpts = {
    origin: '*',
  
    methods: [
      'GET',
      'POST',
    ],
  
    allowedHeaders: [
      'Content-Type',
    ],
  };
  
rest.use(function(req, res, next) {
    res.header("Access-Control-Allow-Origin", "*");
    res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
    next();
 });
  
//Vars
const timeOut = 255; //Connection timeout is disabled if value is set to 255
const listenPort = 4000;
var isOnline = false;
var osramBulbList = []; 
var isConnecting = false;
//OSRAM Lightify Tunable White 60 // A19 Bulb
//Alcance médio da lâmpada com o módulo zigbee Texas Instruments CC2531
//de 3 a 4 metros com visada direta da lâmpada para o módulo.
//Device id: 0x84182600000203a4

var lockList = []


function pingDevices(){
    if(osramBulbList.length !== 0){
        for (lamps in osramBulbList){
            osramBulbList[lamps].ep.read('genOnOff', 'toggle', function (err, data) {
                if(!err){
                    osramBulbList[lamps].online = true;
                }else{
                    osramBulbList[lamps].online = false;
                }
            });
            osramBulbList[lamps].ep.read('genOnOff', 'onOff', function (err, data){
                if(!err)
                    if(data === 1)
                        osramBulbList[lamps].isOn = true;
                    else
                        osramBulbList[lamps].isOn = false;
            });
        }
    }
    if(lockList.length !== 0){
        //TODO: Ping this device a LOT less. It uses batteries, otherwise they will be drained pretty hard by this.
        lockList[0].ep.read('closuresDoorLock', "lockState", function(err, data){
            if(!err){
                lockList[0].online = false
            }else{
                lockList[1].online = true
            }
        })
    }
}

function setLampBrightnessLow(){
    if(osramBulbList.length !== 0){
        for (lamps in osramBulbList){
            osramBulbList[lamps].ep.functional('genLevelCtrl', 'moveToLevel', {'level': 17, 'transtime': 5},function (err, data) {
                if(err){
                }else{
                }
            });
        }
    }
}

//Server is ready
zserver.on('ready', function () {
    console.log('[zServer]Zigbee server is ready! Allow devices to join the network within '+timeOut+' secs.');
    console.log('[zServer]Waiting for incoming clients or messages...');
    zserver.permitJoin(timeOut);
    //Le informacoes do dispositivo como keepalive
    setInterval(pingDevices, 3000);
    setInterval(setLampBrightnessLow, 8000);
});

//While joining is allowed
zserver.on('permitJoining', function (joinTimeLeft) {
    if(joinTimeLeft % 30 == 0){
        console.log('[zServer]Network join timer: '+joinTimeLeft);
    }
});

//Status switch
zserver.on('ind', function (msg) {
    switch (msg.type) {
        case 'devIncoming':
            //devIncoming: novo dispositivo quer se associar ao servidor
            console.log('[zServer]Device: ' + msg.data + ' joining the network!');
            exec('play -q /home/linaro/google_coffee_machine_demo/DB/zbserver/pair.mp3', (err, stdout, stderr) => {
                if (err) {
                    //node couldn't execute the command
                    return;
                }
            });
            isConnecting = false;
            msg.endpoints.forEach(function (device) {
                //Verifica a id do dispositivo
                console.log(device.device.manufName);
                if (device.device.modelId === "LIGHTIFY A19 Tunable White"){
                    osramBulbList.push({'ID': device.device.manufId, 'ep': device, 'online': true, 'isOn': true});
                    console.log(osramBulbList)
                    console.log("[zServer] Found OSRAM Bulb!");
                    setLampBrightnessLow()
                    isOnline = true;
                } else if(device.device.manufName === "Yale") {
                    console.log('[zServer]Yale zigbee lock encontrado!');
                    lockList.push({'ID': "LOCKENDPOINT", 'ep': device, 'online':true});
                    console.log(device)
                }
            });
            break;

        case 'devInterview':
            console.log("Callback > devInterview = Dispositivo se associando")
            if(isConnecting === false){
                exec('aplay /home/linaro/google_coffee_machine_demo/DB/zbserver/dev_connected.wav', (err, stdout, stderr) => {
                    if (!err) {
                        exec('aplay /home/linaro/google_coffee_machine_demo/DB/zbserver/dev_connected.wav', (err, stdout, stderr) => {
                            if (err) {
                                //node couldn't execute the command
                                return;
                            }
                        });
                    }else{
                        return;
                    }
                });
                isConnecting = true;
            }
            break;

        default:
            console.log('Tipo do callback não definido! > '+msg.type);
            break;
    }
});

//Desativa a lâmpada ao receber um post request
//turnOff?devId=1234
rest.post('/turnOff', function(req, res){
    deviceId = req.query.devId;
    console.log("Requested device id is "+deviceId.toString());

    //Analisa se a lâmpada foi conectada corretamente
    if(osramBulbList.length !== 0){
        console.log("[zServer]Recebido POST request para ligar a lâmpada!")
        console.log(osramBulbList)
        found = false;
        for (lamps in osramBulbList){
            console.log(osramBulbList[lamps].ID.toString())
            if(osramBulbList[lamps].ID.toString() === deviceId.toString()){
                found = true
                osramBulbList[lamps].ep.functional('genOnOff', 'off', {}, function (err) {
                    if(!err){
                        osramBulbList[lamps].online = true
                        osramBulbList[lamps].isOn = false
                        res.send(true);
                    }else{
                        osramBulbList[lamps].online = false
                        osramBulbList[lamps].isOn = false
                        res.send(false);
                    }
                });
            }
        }
        if(!found){
            console.log("No lamps found with given id.");
            res.send(false);
        }
    }else{
        res.send(false);
    }
});

rest.post('/unlockDoor', function(req, res){
    //Analisa se a lâmpada foi conectada corretamente
    console.log("[zServer]Recebido GET request para fechadura!")
    
    lockList[0].ep.functional('closuresDoorLock', 'unlockDoor', {'pincodevalue': '\u0001\u0002\u0003\u0004'}, function (err, rsp) {
        if(!err){
            res.send(true);
        }else{
            res.send(false);
        }
    });
});

rest.post('/lockDoor', function(req, res){
    //Analisa se a lâmpada foi conectada corretamente
    console.log("[zServer]Recebido GET request para fechadura!")
    
    lockList[0].ep.functional('closuresDoorLock', 'lockDoor', {'pincodevalue': '\x01\x02\x03\x04'}, function (err, rsp) {
        if(!err){
            res.send(true);
        }else{
            res.send(false);
        }
    });
});

rest.get('/doorState', function(req, res){
    //Retorna o estado atual da fechadura, onde 1 é trancado e 2 é destrancado
    console.log("[zServer]Recebido GET request para estado da fechadura!")
    
    lockList[0].ep.read('closuresDoorLock', 'lockState', function (err, data) {
        if(!err){
            res.send(data.toString());
        }else{
            res.send(data.toString());
        }
    });
});


//Ativa a lâmpada ao receber um post request
//turnOn?devId=1234
rest.post('/turnOn', function(req, res){
    //Analisa se a lâmpada foi conectada corretamente
    deviceId = req.query.devId;
    console.log("Requested device id is "+deviceId.toString())

    if(osramBulbList.length !== 0){
        console.log("[zServer]Recebido POST request para ligar a lâmpada!")
        console.log(osramBulbList)
        found = false
        for (lamps in osramBulbList){
            console.log(osramBulbList[lamps].ID.toString())
            if(osramBulbList[lamps].ID.toString() === deviceId.toString()){
                found = true
                osramBulbList[lamps].ep.functional('genOnOff', 'on', {}, function (err) {
                    if(!err){
                        osramBulbList[lamps].online = true
                        osramBulbList[lamps].isOn = true
                        res.send(true);
                    }else{
                        osramBulbList[lamps].online = false
                        osramBulbList[lamps].isOn = false
                        res.send(false);
                    }
                });
            }
        }
        if(!found){
            console.log("No lamps found with given id.")
            res.send(false)
        }
    }else{
        res.send(false);
    }
});

//Inicia o servidor de zigbee
zserver.start(function (err) {
    console.log("Init ZShepherd")
    if (err)
        console.log(err);
});

//Get de teste
rest.get('/', function(req, res){
    res.send("ZServer is running.")
});

rest.get('/deviceState', function (req, res) {
    message = []
    if(osramBulbList.length !== 0){
        for (lamps in osramBulbList){
            message.push({'devId': osramBulbList[lamps].ID, 'online': osramBulbList[lamps].online, 'isOn': osramBulbList[lamps].isOn})
        }
    }
    if(lockList.length !== 0){
        for (locks in lockList){
            message.push({'devId': lockList[locks].ID, 'online': lockList[locks].online})
        }
    }
    res.setHeader('Content-Type', 'application/json');
    res.send(JSON.stringify({message}));
})

//Inicia o listener para receber POSTs e GETs
rest.listen(listenPort);
