from google.cloud import pubsub_v1
import logging
import json
from time import sleep
import time

class PBClient:
	def __init__(self):
		self.publisher = None
		self.subscriber = None
		self.projectId = None

def getClient(projectId):
	#print("[PBCtl] Creating pubsub client for project ID "+str(projectId))
	#Creates a Publisher and a Subscriber and puts in PBClient object
	client = PBClient()
	logging.basicConfig()

	#WARNING: Do not use explicit credential declaration below!
	client.publisher = pubsub_v1.PublisherClient()
	client.subscriber = pubsub_v1.SubscriberClient()
	client.projectId = projectId
	return client

def get_utc_timestamp(seconds=None):
    return time.strftime("%Y-%m-%dT%H:%M:%S.00Z", time.gmtime(seconds))

client = getClient("db410c-dev-ga04-233012")

lampStatus = None
coffeeMachineStatus = None

#Bluetooth lamp JSON payloads
payload_lamp_on = {
    'cmd': 'on',
    'dev': 'lamp'
}
payload_lamp_off = {
    'cmd': 'off',
    'dev': 'lamp'
}

#Coffee machine JSON payloads
payload_short_coffee = {
    'cmd': 'shortCoffee'
}
payload_long_coffee = {
    'cmd': 'longCoffee'
}
payload_coffee_on = {
    'cmd': 'on',
    'dev': 'coffee'
}
payload_coffee_off = {
    'cmd': 'off',
    'dev': 'coffee'
}
payload_coffee_status = {
    'cmd': 'status',
    'dev': 'coffee'
}

#Zigbee and Locker payloads
payload_unlock = {
    'cmd': 'unlock'
}
payload_lock = {
    'cmd': 'lock'
}
payload_zigbee_on = {
    'cmd': 'lighton'
}
payload_zigbee_off = {
    'cmd': 'lightoff'
}

def zigCtlControlCallback(message):
    global lampStatus

    message.ack()
    print("[Dash Zigbee] Message: " + str(message.data))
    msg_json = json.loads(message.data)
    if(msg_json['cmd']):
        if(msg_json['cmd'] == 'lightoff'):
            lampStatus = 'lightoff'
        elif(msg_json['cmd'] == 'lighton'):
            lampStatus = 'lighton'

def bluDevControlCallback(message):   
    global coffeeMachineStatus

    message.ack()
    print("[Dash Bluetooth] Message: " + str(message.data))
    msg_json = json.loads(message.data)
    if 'cmd' in msg_json:
        if(msg_json['cmd'] == 'on'):
            if(msg_json['dev'] == 'coffee'):
                pass
                #print("coffeeOn")
        elif(msg_json['cmd'] == 'off'):
            if(msg_json['dev'] == 'coffee'):
                pass
                #print("coffeeOff")
        elif(msg_json['cmd'] == 'shortCoffee'):
            pass
            #print("shortCoffee")
        elif(msg_json['cmd'] == 'longCoffee'):
            pass
            #print("longCoffee")
    elif 'isOn' in msg_json:
        coffeeMachineStatus = message.data

def subscribeTopics(client):
    #Topics (Only subscribe to subscription topics set in GCP!)
    TOPIC_ZIGCTL_SUB = 'projects/{project_id}/subscriptions/DashZigSub'.format(project_id=client.projectId)
    TOPIC_BLUDEVCTL_SUB = 'projects/{project_id}/subscriptions/DashBluDevSub'.format(project_id=client.projectId)

    #Subscription routine
    zig = client.subscriber.subscribe(TOPIC_ZIGCTL_SUB, zigCtlControlCallback)
    blu = client.subscriber.subscribe(TOPIC_BLUDEVCTL_SUB, bluDevControlCallback)
    
    print("[Dash Ctl] Subscription complete.")

def turnOnCoffeeMachine():
    global client
    payload_coffee_on['time'] = get_utc_timestamp()
    client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectId), json.dumps(payload_coffee_on).encode('utf-8'))

def turnOffCoffeeMachine():
    global client
    payload_coffee_off['time'] = get_utc_timestamp()
    client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectId), json.dumps(payload_coffee_off).encode('utf-8'))

def getCoffeeMachineStatus():    
    global coffeeMachineStatus
    payload_coffee_status['time'] = get_utc_timestamp()
    client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectId), json.dumps(payload_coffee_status).encode('utf-8'))
    #print("Coffee Machine Status: " + str(coffeeMachineStatus))
    return coffeeMachineStatus

def makeLongCoffee():
    payload_long_coffee['time'] = get_utc_timestamp()
    client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectId), json.dumps(payload_long_coffee).encode('utf-8'))

def makeShortCoffee():
    payload_short_coffee['time'] = get_utc_timestamp()
    client.publisher.publish('projects/{project_id}/topics/BluDevCtl'.format(project_id=client.projectId), json.dumps(payload_short_coffee).encode('utf-8'))

def SubTopics():
    subscribeTopics(client)

def getMainLampStatus():
    global lampStatus
    return lampStatus

def turnOnMainLamp():
    payload_zigbee_on['time'] = get_utc_timestamp()
    client.publisher.publish('projects/{project_id}/topics/ZigCtl'.format(project_id=client.projectId), json.dumps(payload_zigbee_on).encode('utf-8'))

def turnOffMainLamp():
    payload_zigbee_off['time'] = get_utc_timestamp()
    client.publisher.publish('projects/{project_id}/topics/ZigCtl'.format(project_id=client.projectId), json.dumps(payload_zigbee_off).encode('utf-8'))